Class: SugarJar::Commands

Inherits:
Object
  • Object
show all
Includes:
Util
Defined in:
lib/sugarjar/commands.rb

Overview

This is the workhorse of SugarJar. Short of #initialize, all other public methods are “commands”. Anything in private is internal implementation details.

Constant Summary collapse

MAIN_BRANCHES =
%w{master main}.freeze

Instance Method Summary collapse

Methods included from Util

#gh, #gh_nofail, #git, #git_nofail, #hub, #hub_nofail, #in_repo, #repo_name, #repo_root, #which, #which_nofail

Constructor Details

#initialize(options) ⇒ Commands

Returns a new instance of Commands.



17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
# File 'lib/sugarjar/commands.rb', line 17

def initialize(options)
  SugarJar::Log.debug("Commands.initialize options: #{options}")
  @ghuser = options['github_user']
  @ghhost = options['github_host']
  @cli = options['github_cli']
  @ignore_dirty = options['ignore_dirty']
  @ignore_prerun_failure = options['ignore_prerun_failure']
  @repo_config = SugarJar::RepoConfig.config
  @color = options['color']
  @checks = {}
  @main_branch = nil
  @main_remote_branches = {}
  return if options['no_change']

  set_hub_host
  set_commit_template if @repo_config['commit_template']
end

Instance Method Details

#amend(*args) ⇒ Object



141
142
143
144
145
# File 'lib/sugarjar/commands.rb', line 141

def amend(*args)
  assert_in_repo
  # This cannot use shellout since we need a full terminal for the editor
  exit(system(which('git'), 'commit', '--amend', *args))
end

#bclean(name = nil) ⇒ Object



49
50
51
52
53
54
55
56
57
58
59
60
# File 'lib/sugarjar/commands.rb', line 49

def bclean(name = nil)
  assert_in_repo
  name ||= current_branch
  if clean_branch(name)
    SugarJar::Log.info("#{name}: #{color('reaped', :green)}")
  else
    die(
      "#{color("Cannot clean #{name}", :red)}! there are unmerged " +
      "commits; use 'git branch -D #{name}' to forcefully delete it.",
    )
  end
end

#bcleanallObject



62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
# File 'lib/sugarjar/commands.rb', line 62

def bcleanall
  assert_in_repo
  curr = current_branch
  all_local_branches.each do |branch|
    if MAIN_BRANCHES.include?(branch)
      SugarJar::Log.debug("Skipping #{branch}")
      next
    end

    if clean_branch(branch)
      SugarJar::Log.info("#{branch}: #{color('reaped', :green)}")
    else
      SugarJar::Log.info("#{branch}: skipped")
      SugarJar::Log.debug(
        "There are unmerged commits; use 'git branch -D #{branch}' to " +
        'forcefully delete it)',
      )
    end
  end

  # Return to the branch we were on, or main
  if all_local_branches.include?(curr)
    git('checkout', curr)
  else
    checkout_main_branch
  end
end

#binfoObject



101
102
103
104
105
106
107
# File 'lib/sugarjar/commands.rb', line 101

def binfo
  assert_in_repo
  SugarJar::Log.info(git(
    'log', '--graph', '--oneline', '--decorate', '--boundary',
    "#{tracked_branch}.."
  ).stdout.chomp)
end

#brObject



96
97
98
99
# File 'lib/sugarjar/commands.rb', line 96

def br
  assert_in_repo
  SugarJar::Log.info(git('branch', '-v').stdout.chomp)
end

#co(*args) ⇒ Object



90
91
92
93
94
# File 'lib/sugarjar/commands.rb', line 90

def co(*args)
  assert_in_repo
  s = git('checkout', *args)
  SugarJar::Log.info(s.stderr + s.stdout.chomp)
end

#feature(name, base = nil) ⇒ Object



35
36
37
38
39
40
41
42
43
44
45
46
47
# File 'lib/sugarjar/commands.rb', line 35

def feature(name, base = nil)
  assert_in_repo
  SugarJar::Log.debug("Feature: #{name}, #{base}")
  die("#{name} already exists!") if all_local_branches.include?(name)
  base ||= most_main
  base_pieces = base.split('/')
  git('fetch', base_pieces[0]) if base_pieces.length > 1
  git('checkout', '-b', name, base)
  SugarJar::Log.info(
    "Created feature branch #{color(name, :green)} based on " +
    color(base, :green),
  )
end

#forcepush(remote = nil, branch = nil) ⇒ Object Also known as: fpush



260
261
262
263
# File 'lib/sugarjar/commands.rb', line 260

def forcepush(remote = nil, branch = nil)
  assert_in_repo
  _smartpush(remote, branch, true)
end

#lintObject



243
244
245
246
# File 'lib/sugarjar/commands.rb', line 243

def lint
  assert_in_repo
  exit(1) unless run_check('lint')
end

#pullsuggestionsObject Also known as: ps



297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
# File 'lib/sugarjar/commands.rb', line 297

def pullsuggestions
  assert_in_repo

  if dirty?
    if @ignore_dirty
      SugarJar::Log.warn(
        'Your repo is dirty, but --ignore-dirty was specified, so ' +
        'carrying on anyway.',
      )
    else
      SugarJar::Log.error(
        'Your repo is dirty, so I am not going to push. Please commit ' +
        'or amend first.',
      )
      exit(1)
    end
  end

  src = "origin/#{current_branch}"
  fetch('origin')
  diff = git('diff', src).stdout
  return unless diff && !diff.empty?

  puts "Will merge the following suggestions:\n\n#{diff}"

  loop do
    $stdout.print("\nAre you sure? [y/n] ")
    ans = $stdin.gets.strip
    case ans
    when /^[Yy]$/
      system(which('git'), 'merge', '--ff', "origin/#{current_branch}")
      break
    when /^[Nn]$/, /^[Qq](uit)?/
      puts 'Not merging at user request...'
      break
    else
      puts "Didn't understand '#{ans}'."
    end
  end
end

#qamend(*args) ⇒ Object Also known as: amendq



147
148
149
150
# File 'lib/sugarjar/commands.rb', line 147

def qamend(*args)
  assert_in_repo
  SugarJar::Log.info(git('commit', '--amend', '--no-edit', *args).stdout)
end

#smartclone(repo, dir = nil, *args) ⇒ Object Also known as: sclone



176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
# File 'lib/sugarjar/commands.rb', line 176

def smartclone(repo, dir = nil, *args)
  # If the user has specified a hub host, set the environment variable
  # since we don't have a repo to configure yet
  ENV['GITHUB_HOST'] = @ghhost if @ghhost

  reponame = File.basename(repo, '.git')
  dir ||= reponame

  SugarJar::Log.info("Cloning #{reponame}...")

  # GH's 'fork' command (with the --clone arg) will fork, if necessary,
  # then clone, and then setup the remotes with the appropriate names. So
  # we just let it do all the work for us and return.
  if gh?
    ghcli('repo', 'fork', '--clone', canonicalize_repo(repo), dir, *args)
    SugarJar::Log.info('Remotes "origin" and "upstream" configured.')
    return
  end

  # For 'hub', first we clone, using git, as 'hub' always needs a repo
  # to operate on.
  git('clone', canonicalize_repo(repo), dir, *args)

  # Then we go into it and attempt to use the 'fork' capability
  Dir.chdir dir do
    # Now that we have a repo, if we have a hub host set it.
    set_hub_host

    org = extract_org(repo)
    SugarJar::Log.debug("Comparing org #{org} to ghuser #{@ghuser}")
    if org == @ghuser
      puts 'Cloned forked or self-owned repo. Not creating "upstream".'
      SugarJar::Log.info('Remotes "origin" and "upstream" configured.')
      return
    end

    s = ghcli_nofail('repo', 'fork', '--remote-name=origin')
    if s.error?
      if s.stdout.include?('SAML enforcement')
        SugarJar::Log.info(
          'Forking the repo failed because the repo requires SAML ' +
          "authentication. Full output:\n\n\t#{s.stdout}",
        )
        exit(1)
      else
        # gh as well as old versions of hub, it would fail if the upstream
        # fork already existed. If we got an error, but didn't recognize
        # that, we'll assume that's what happened and try to add the remote
        # ourselves.
        SugarJar::Log.info("Fork (#{@ghuser}/#{reponame}) detected.")
        SugarJar::Log.debug(
          'The above is a bit of a lie. "hub" failed to fork and it was ' +
          'not a SAML error, so our best guess is that a fork exists ' +
          'and so we will try to configure it.',
        )
        git('remote', 'rename', 'origin', 'upstream')
        git('remote', 'add', 'origin', forked_repo(repo, @ghuser))
      end
    else
      SugarJar::Log.info("Forked #{reponame} to #{@ghuser}")
    end
    SugarJar::Log.info('Remotes "origin" and "upstream" configured.')
  end
end

#smartlogObject Also known as: sl

binfo for all branches



110
111
112
113
114
115
116
# File 'lib/sugarjar/commands.rb', line 110

def smartlog
  assert_in_repo
  SugarJar::Log.info(git(
    'log', '--graph', '--oneline', '--decorate', '--boundary',
    '--branches', "#{most_main}.."
  ).stdout.chomp)
end

#smartpullrequest(*args) ⇒ Object Also known as: spr, smartpr



275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
# File 'lib/sugarjar/commands.rb', line 275

def smartpullrequest(*args)
  assert_in_repo
  assert_common_main_branch
  if dirty?
    SugarJar::Log.warn(
      'Your repo is dirty, so I am not going to create a pull request. ' +
      'You should commit or amend and push it to your remote first.',
    )
    exit(1)
  end
  if gh?
    SugarJar::Log.trace("Running: gh pr create #{args.join(' ')}")
    system(which('gh'), 'pr', 'create', *args)
  else
    SugarJar::Log.trace("Running: hub pull-request #{args.join(' ')}")
    system(which('hub'), 'pull-request', *args)
  end
end

#smartpush(remote = nil, branch = nil) ⇒ Object Also known as: spush



253
254
255
256
# File 'lib/sugarjar/commands.rb', line 253

def smartpush(remote = nil, branch = nil)
  assert_in_repo
  _smartpush(remote, branch, false)
end

#unitObject



248
249
250
251
# File 'lib/sugarjar/commands.rb', line 248

def unit
  assert_in_repo
  exit(1) unless run_check('unit')
end

#upObject



120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
# File 'lib/sugarjar/commands.rb', line 120

def up
  assert_in_repo
  # get a copy of our current branch, if rebase fails, we won't
  # be able to determine it without backing out
  curr = current_branch
  result = gitup
  if result['so'].error?
    die(
      "#{color(curr, :red)}: Failed to rebase on " +
      "#{result['base']}. Leaving the repo as-is. You can get out of " +
      'this with a `git rebase --abort`. Output from failed rebase is: ' +
      "\nSTDOUT:\n#{result['so'].stdout.lines.map { |x| "\t#{x}" }.join}" +
      "\nSTDERR:\n#{result['so'].stderr.lines.map { |x| "\t#{x}" }.join}",
    )
  else
    SugarJar::Log.info(
      "#{color(current_branch, :green)} rebased on #{result['base']}",
    )
  end
end

#upallObject



154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
# File 'lib/sugarjar/commands.rb', line 154

def upall
  assert_in_repo
  all_local_branches.each do |branch|
    next if MAIN_BRANCHES.include?(branch)

    git('checkout', branch)
    result = gitup
    if result['so'].error?
      SugarJar::Log.error(
        "#{color(branch, :red)} failed rebase. Reverting attempt and " +
        'moving to next branch. Try `sj up` manually on that branch.',
      )
      git('rebase', '--abort')
    else
      SugarJar::Log.info(
        "#{color(branch, :green)} rebased on " +
        color(result['base'], :green).to_s,
      )
    end
  end
end

#versionObject



267
268
269
270
271
272
273
# File 'lib/sugarjar/commands.rb', line 267

def version
  puts "sugarjar version #{SugarJar::VERSION}"
  puts ghcli('version').stdout
  # 'hub' prints the 'git' version, but gh doesn't, so if we're on 'gh'
  # print out the git version directly
  puts git('version').stdout if gh?
end